#include <P33FXXXX.h>
#include <uart.h>
#include <spi.h>
#include <math.h>
#include "my_define.h"

//Macros for Configuration Fuse Registers (copied from device header file):
//_FOSC(CSW_FSCM_OFF & XT_PLL8);  //Set up for crystal multiplied by 8x PLL
//_FWDT(WDT_OFF);                 //Turn off the Watch-Dog Timer.
//_FBORPOR(MCLR_EN & PWRT_OFF);   //Enable MCLR reset pin and turn off the power-up timers.
//_FGS(CODE_PROT_OFF);            //Disable Code Protection


int	__attribute__  (( address ( 0x4000)))  ADCDMA[10] ;

#define	ENABLE_PWM()		OC6CON &= 0xfff8 ; OC6CON |= 0x0006
#define	DISABLE_PWM()		OC6CON &= 0xfff8

union
	{
		unsigned int	WordValue ;
		struct
		{
			unsigned char	LowByte ;
			unsigned char	HighByte ;
		};
	}Word2Byte ;

unsigned 		SinTable[120];
int				SinIndex = 0 ;
int				SinIndex1 = 0 ;
unsigned 		TotalLoop = 0 ;
unsigned char 	ASCII_Buf[10] ;
unsigned		T2_500ms ;
unsigned		T2_AccuCounter = 0 ;
unsigned 		Value_VR1 , Value_VR2 ,Value_SW124;
unsigned 		QEI_Old , QEI_New ;
unsigned		QEI_Update = 0 ;
unsigned		QEI_SampleTime = 0 ;
unsigned 		PWM_Time= 0 ;

extern int 		__C30_UART ;

void			hexitoa(int  , unsigned char *) ;
unsigned char	SPI_ByteWrite(unsigned char );
void			SPI_DAC_Write(unsigned char , unsigned int );
void			uitoa( unsigned char *, unsigned int );
void			Get_SinTable(void);
void			Show_ADC(void) ;
void			Show_RPM(void) ;
void			Show_SW(void) ;


void	_ISR	_DMA0Interrupt(void)
{
		Value_VR1 = ADCDMA[1] ;
		Value_VR2 = ADCDMA[2] ;
		Value_SW124 = ADCDMA[0] ;
		IFS0bits.DMA0IF = 0 ; 
}

void 	_ISR 	_T1Interrupt(void)
{
		IFS0bits.T1IF = 0 ;
		SPI_DAC_Write(0,SinTable[SinIndex]) ;
		SPI_DAC_Write(1,SinTable[SinIndex1]) ;
			
				SinIndex ++ ;
				SinIndex1 += 2 ;
				if ( SinIndex >= 100)	SinIndex = 0 ;
				if ( SinIndex1 >= 100 )	SinIndex1 = 0 ;
}

void	_ISR	_T2Interrupt(void)
{
		IFS0bits.T2IF = 0 ;
		T2_AccuCounter ++ ;
		QEI_SampleTime ++ ;
		if ( QEI_SampleTime >= 20 )
			{
				QEI_Old = QEI_New ;
				QEI_New = ReadQEI( ) ;
				QEI_Update = 1 ;
				QEI_SampleTime = 0 ;
			}
		if ( T2_AccuCounter == 100 )				// 100 * 5 ms = 500 ms	
			{
				T2_AccuCounter = 0 ;
				T2_500ms = 1 ;
			}

		if (  PWM_Time > 0 )
			{
				PWM_Time-- ;

				if ( PWM_Time == 0 )	
					 DISABLE_PWM();

			}
}


int main ( void )
{
	 	//	The settings below set up the oscillator and PLL for 16 MIPS as
        //	follows:
        //            	Crystal Frequency  * (DIVISOR+2)
        //	 Fcy =    ------------------------------------------------
        //              	PLLPOST * (PRESCLR+2) * 2	

        	PLLFBD = 0x009E;				//	0x9E = 158
	      	CLKDIV = 0x0048;				// 	PLLPOST = 01 > Output / 4   & PLLPRE = 8 > Presclaer of PLL = 8+2 = 10

			SRbits.IPL = 0 ;

//		PLLFBDbits.PLLDIV = 6 ;
//		CLKDIVbits.PLLPRE = 0 ;
//		CLKDIVbits.PLLPOST = 1 ;

	Get_SinTable() ;

	Initial_UART1( ) ;
	Initial_UART2( ) ;
	Initial_SPI() ;
	Initial_TMR1() ;
	Initial_TMR2() ;
	Initial_TMR3() ;
	Initial_AD1_DMA0();
	Initial_QEI() ;
	Initial_OC6() ;

	DISABLE_PWM() ;

	QEI_Old = 0 ;
	QEI_New = 0 ;

	OpenLCD() ;
	putrsLCD("RPM= ") ;
	setcurLCD(0,1) ;
	putrsLCD("My ADC= ") ;

	DIR_LED = OUTPUT ;
	DIR_SW5 = INPUT ;
	DIR_SW6 = INPUT ;

	__C30_UART = 1 ;



	while (1) 
	{
		if ( T2_500ms )
		{
			__C30_UART = 1 ;
			printf ( " Hello ! ڬO dsPIC33FJ128MC708 from UART1 %d\n\r",TotalLoop++ ) ;
				
				__C30_UART = 2 ;
				printf ( " Hello ! ڬO dsPIC33FJ128MC708 from UART2 %d\n\r",TotalLoop++ ) ;

			Show_ADC() ;
			T2_500ms = 0 ;
			Show_RPM() ;
			Show_SW() ;
		}
	}

}

void	Show_SW(void)
{
		setcurLCD(13,0) ;
		if ( !SW5)
			{
				ENABLE_PWM();
				PWM_Time = 50 ;		
				putrsLCD("SW5") ;
			}
		else if (!SW6)	
			{
				ENABLE_PWM() ;
				PWM_Time = 50 ;
				putrsLCD("SW6") ;
			}
		else			putrsLCD("SW?") ;

		setcurLCD(13,0) ;

		if ( Value_SW124 < 0x3F0 )			//	If input is 3.3V -> no SW pressed !
			{
				ENABLE_PWM();
				PWM_Time = 50 ;

				if ( Value_SW124 > 800 )	
						putrsLCD("SW4") ;
				else if (Value_SW124 < 800 && Value_SW124 > 670 )
						putrsLCD("SW3")	;
				else if (Value_SW124 < 670 && Value_SW124 > 500 )
						putrsLCD("SW2") ;
				else	putrsLCD("SW1") ;
				
			}
}

void	Show_RPM(void)
{

		unsigned long 	QEI_Diff ;
		unsigned		RPM ;

			setcurLCD(5,0) ;

			if ( QEICONbits.UPDN )		putcLCD('+') ;			// ھUPDN줸Mw઺V
			else						putcLCD('-') ;
					
				
			if ( QEICONbits.UPDN )								// Fɪp
				{
					if ( QEI_New >= QEI_Old )					// pɶƶq
						QEI_Diff = QEI_New - QEI_Old ;
					else										// ɪBz
						QEI_Diff = QEI_New + ( 65536 - QEI_Old ) ;	
				}
			else													// Fɪp
				{
					if (QEI_New < QEI_Old )
						QEI_Diff = QEI_Old - QEI_New ;
					else
						QEI_Diff = ( 65536 - QEI_New )+ QEI_Old ; 	// ɦɪBz
				}
				
			RPM = ((long)QEI_Diff*600/256);			// tp (100mS * 600= 1 Minute, Index=64 4 = 256)

					uitoa ( ASCII_Buf , RPM ) ;		// ഫQiAন ASCII Code

					setcurLCD(7,0) ;				// tb LCD 
					putrsLCD( "     " ) ;

					setcurLCD(7,0) ;
					putsLCD( ASCII_Buf ) ;

}

void	Show_ADC(void)
{
			setcurLCD(7,1) ;
			hexitoa( Value_VR1 , ASCII_Buf);
			putrsLCD("          ") ;
				setcurLCD(7,1) ;		
					putsLCD(ASCII_Buf) ;

			putrsLCD(" ") ;
			hexitoa( Value_VR2 , ASCII_Buf);	
				putsLCD(ASCII_Buf) ;
}

void	Get_SinTable(void)
{
	int	LoopVar1 , TableIndex ;

	for ( LoopVar1 = 0 ; LoopVar1 < 100 ; LoopVar1 ++ )
	{
		TableIndex = LoopVar1 * 3.6 ;
		SinTable[LoopVar1] = (sin( (3.1415926 * TableIndex) / (double)180 ) + 1 ) * HALF_DUTY  ;
	}
}


void	uitoa( unsigned char *ASCII_Buf , unsigned int IntegerValue )
{
	
	unsigned int  TempValue ;
	unsigned char ZeroDisable ;
	unsigned int  BaseQty ;
	unsigned int  Loop ;

	ZeroDisable = 1 ;
	BaseQty = 10000 ;

	for ( Loop = 0 ; Loop < 4 ; Loop ++)
	{
		TempValue = IntegerValue / BaseQty ;

			if 	( TempValue > 0) 	
				{
					*ASCII_Buf++ = (unsigned char)TempValue + '0' ;
					ZeroDisable = 0 ;
				}
			else if	( ZeroDisable == 0 ) 
					*ASCII_Buf++ = '0' ;
		IntegerValue = IntegerValue - ( TempValue * BaseQty ) ;
		BaseQty = BaseQty / 10 ;

	}
		*ASCII_Buf++ = (unsigned char)IntegerValue + '0' ;
		*ASCII_Buf = (unsigned char) 0x00 ;
		
}

void	hexitoa(int BinaryValue , unsigned char *ResultBuffer) 
{

		unsigned char TempChar1 ;

		TempChar1 = ( BinaryValue >> 12 ) & 0x0f ;
		if ( TempChar1 > 9 )		*ResultBuffer++ = TempChar1 + 'A'-10 ;
		else						*ResultBuffer++ = TempChar1 + '0' ;

		TempChar1 = ( BinaryValue >> 8 )  & 0x0f;
		if ( TempChar1 > 9 )		*ResultBuffer++ = TempChar1 + 'A'-10 ;
		else						*ResultBuffer++ = TempChar1 + '0' ;
		
		TempChar1 = ( BinaryValue >> 4 ) & 0x0f ;
		if ( TempChar1 > 9 )		*ResultBuffer++ = TempChar1 + 'A'-10 ;
		else						*ResultBuffer++ = TempChar1 + '0' ;

		TempChar1 = ( BinaryValue & 0x0f ) ;
		if ( TempChar1 > 9 )		*ResultBuffer++ = TempChar1 + 'A'-10 ;
		else						*ResultBuffer++ = TempChar1 + '0' ;
		
		*ResultBuffer = 0x00 ;
}



void	SPI_DAC_Write(unsigned char CH_NO , unsigned int DAC_Data ) 
{
		unsigned char	Status ;
		unsigned int	SPI_MASK ;

		if ( CH_NO == 0 ) 	SPI_MASK = 0b0111000000000000 ;
		else				SPI_MASK = 0b1111000000000000 ;

		Word2Byte.WordValue = SPI_MASK | DAC_Data ;

		SPI_CS = 0 ;
			
			SPI_ByteWrite ( Word2Byte.HighByte ) ;				// Send Write Command 
			SPI_ByteWrite ( Word2Byte.LowByte ) ;
			
		SPI_CS = 1 ;

		DAC_LOAD = 0 ;
		Nop() ;
		Nop() ;
		DAC_LOAD = 1 ;

}

unsigned char	SPI_ByteWrite(unsigned char SPI_DATA )
{
		int	WaitLoop ;

		SPI1BUF = SPI_DATA ;
		while ( ! IFS0bits.SPI1IF ) ;
			IFS0bits.SPI1IF = 0 ;
			Nop() ;
			Nop() ;
		return (unsigned char) SPI1BUF ;

}
