/*********************************************************************
 *
 *                Microchip USB C18 Firmware Version 1.0
 *
 *********************************************************************
 * FileName:        user.c
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 2.30.01+
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the Company? for its PICmicro?Microcontroller is intended and
 * supplied to you, the Companys customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS?CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Rawin Rojvanit       11/19/04    Original.
 ********************************************************************/

/******************************************************************************
 * CDC RS-232 Emulation Tutorial Instructions:
 ******************************************************************************
 * Refer to Application Note AN956 for explanation of the CDC class.
 * 
 * First take a look at Exercise_Example() and study how functions are called.
 * 
 * There are five exercises, each one has a solution in the CDC\user\solutions.
 * Scroll down and look for Exercise_01,_02,_03,_04, and _05.
 * Instructions on what to do is inside each function.
 * 
 *****************************************************************************/

/** I N C L U D E S **********************************************************/
#include <p18cxxx.h>
#include <usart.h>
#include "system\typedefs.h"

#include "system\usb\usb.h"

#include "io_cfg.h"             // I/O pin mapping
#include "user\user.h"

/** V A R I A B L E S ********************************************************/
#pragma udata
byte old_sw9,old_sw10;

//	char input_buffer[64];
//	char output_buffer[64];
byte VT100_Buffer[10];			// Reserved for VT-100 Command !!

unsigned char Receive_Buffer[64] , UTX_Buffer[64] ;
static	unsigned char Exer1_Done = 0 ;
static	unsigned  char WritePointer = 0 , ReadPointer = 0 ;
unsigned ReadCounter ;

rom char welcome[]={"PIC18F4550 Full-Speed USB - CDC RS-232 Data Transfer Port\r\n\r\n"};
rom char ansi_clrscr[]={"\x1b[2J"};         // ANSI Clear Screen Command

/** P R I V A T E  P R O T O T Y P E S ***************************************/
void InitializeUSART(void);
void BlinkUSBStatus(void);
BOOL Switch9IsPressed(void);
BOOL Switch10IsPressed(void);

void Exercise_Example(void);

void Exercise_01(void);
void Exercise_02(void);
void Exercise_03(void);
void Exercise_04(void);
void Exercise_05(void);

void	VT100_Clr_Right(void) ;
void	VT100_Clr_Line(void) ;
void 	isr_high(void) ;
void 	isr_low(void) ;

#define	ASCII_ESC	0x1b

/** D E C L A R A T I O N S **************************************************/
#pragma code isrhighcode = 0x0008

void isr_high_direct(void)	
{  								
     _asm 		//begin in-line assembly
     goto isr_high	//go to isr_high function
     _endasm 		//end in-line assembly
}
#pragma code

//************************************************
//* Function: isr_high(void)                     *     
//*   High priority interrupt will               *
//*   - Received a serial data from RS-232       *
//*     Save the received data to buffer Rec_Data*                  
//************************************************
#pragma interrupt isr_high 

void isr_high(void)
{
	if ( PIR1bits.RCIF ) 
	{
		Receive_Buffer[WritePointer++] = RCREG ;
		if ( WritePointer >= 64 )	WritePointer = 0 ;	
	}
}	
							
#pragma code

#pragma code isrlowcode = 0x0018

void isr_low_direct(void)	
{  								
     _asm 		//begin in-line assembly
     goto isr_low	//go to isr_high function
     _endasm 		//end in-line assembly
}
#pragma code

#pragma interruptlow isr_low

void isr_low(void)
{
	Nop( ) ;
}								

#pragma code

void UserInit(void)
{
    mInitAllLEDs();
    mInitAllSwitches();
    old_sw9 = sw9;
    old_sw10 = sw10;
    
    InitializeUSART();   

		// 	Initialization for Interrpts !!
		RCONbits.IPEN = 1 ;		//	Enable Priority interrupt type !!
		IPR2bits.USBIP = 0 ;		// 	Set USBIF as Low priority interrupt 
		IPR1bits.RCIP = 1 ;			// 	Set RCIF as High priority interrupt 

		INTCONbits.GIEH = 1 ;
		INTCONbits.GIEL = 0 ;

		PIE1bits.RCIE = 1 ;

}//end UserInit

void InitializeUSART(void)
{
    TRISCbits.TRISC7=1; // RX
    TRISCbits.TRISC6=0; // TX
//    SPBRG = 0xC4;
//    SPBRGH = 0x09;      // 0x09C41 for 48MHz -> 4800 baud
    SPBRG = 0x70;
    SPBRGH = 0x02;      // 0x270 for 48MHz -> 19200 baud

    TXSTA = 0x24;       // TX enable BRGH=1
    RCSTA = 0x90;       // continuous RX
    BAUDCON = 0x08;     // BRG16 = 1
}//end InitializeUSART

/******************************************************************************
 * Function:        void ProcessIO(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is a place holder for other user routines.
 *                  It is a mixture of both USB and non-USB tasks.
 *
 * Note:            None
 *****************************************************************************/
void ProcessIO(void)
{   
    BlinkUSBStatus();
    // User Application USB tasks
    if((usb_device_state < CONFIGURED_STATE)||(UCONbits.SUSPND==1)) return;


		if ( RCSTAbits.FERR )	RCSTAbits.FERR = 0 ;
			if ( RCSTAbits.OERR) 	
			{
				RCSTAbits.CREN = 0  ;

					Nop( ) ;

				RCSTAbits.CREN = 1 ;

			}

		if ( (ReadPointer != WritePointer) && mUSBUSARTIsTxTrfReady( )  )
			{
				ReadCounter = 0 ;
				while ( ReadPointer != WritePointer )
				{
					UTX_Buffer[ ReadCounter++ ] = Receive_Buffer [ReadPointer++] ;
					if ( ReadPointer >= 64 )		ReadPointer = 0 ;
				}

						mUSBUSARTTxRam(UTX_Buffer , ReadCounter ) ;

			}

}	//	End ProcessIO


/******************************************************************************
 * Function:        void BlinkUSBStatus(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        BlinkUSBStatus turns on and off LEDs corresponding to
 *                  the USB device state.
 *
 * Note:            mLED macros can be found in io_cfg.h
 *                  usb_device_state is declared in usbmmap.c and is modified
 *                  in usbdrv.c, usbctrltrf.c, and usb9.c
 *****************************************************************************/
void BlinkUSBStatus(void)
{
    static word led_count=0;
    
    if(led_count == 0)led_count = 10000U;
    led_count--;

    #define mLED_Both_Off()         {mLED_13_Off();mLED_14_Off();}
    #define mLED_Both_On()          {mLED_13_On();mLED_14_On();}
    #define mLED_Only_13_On()        {mLED_13_On();mLED_14_Off();}
    #define mLED_Only_14_On()        {mLED_13_Off();mLED_14_On();}

    if(UCONbits.SUSPND == 1)
    {
        if(led_count==0)
        {
            mLED_13_Toggle();
            mLED_14 = mLED_13;        // Both blink at the same time
        }//end if
    }
    else
    {
        if(usb_device_state == DETACHED_STATE)
        {
            mLED_Both_Off();
        }
        else if(usb_device_state == ATTACHED_STATE)
        {
            mLED_Both_On();
        }
        else if(usb_device_state == POWERED_STATE)
        {
            mLED_Only_13_On();
        }
        else if(usb_device_state == DEFAULT_STATE)
        {
            mLED_Only_14_On();
        }
        else if(usb_device_state == ADDRESS_STATE)
        {
            if(led_count == 0)
            {
                mLED_13_Toggle();
                mLED_14_Off();
            }//end if
        }
        else if(usb_device_state == CONFIGURED_STATE)
        {
            if(led_count==0)
            {
                mLED_13_Toggle();
                mLED_14 = !mLED_13;       // Alternate blink                
            }//end if
        }//end if(...)
    }//end if(UCONbits.SUSPND...)

}//end BlinkUSBStatus

BOOL Switch9IsPressed(void)
{
    if(sw9 != old_sw9)
    {
        old_sw9 = sw9;                  // Save new value
        if(sw9 == 0)                    // If pressed
            return TRUE;                // Was pressed
    }//end if
    return FALSE;                       // Was not pressed
}//end Switch2IsPressed

BOOL Switch10IsPressed(void)
{
    if(sw10 != old_sw10)
    {
        old_sw10 = sw10;                  // Save new value
        if(sw10 == 0)                    // If pressed
            return TRUE;                // Was pressed
    }//end if
    return FALSE;                       // Was not pressed
}//end Switch3IsPressed

/** EOF user.c ***************************************************************/
